#!/usr/bin/perl -w

package GOGraph::Crawl::CrawlToRoot;


=head1 NAME

  GOGraph::Crawl::CrawlToRoot

=head1 SYNOPSIS

  my $crawl = GOGraph::Crawl::CrawlToRoot->new( GOGraph::Crawl::AcceptAll->new(),
                                                3674 );

  my $terms = $crawl->crawl_to_root( $start_term );

=head1 DESCRIPTION

  This class allows crawling around the GO graph, from a given node towards
  the "root". In this case this means either the actual root node, or another
  defined root.

  The main point of this class is that it fiddles with the graph as it goes,
  by controlling the kind of link up which it is allowed to travel. This
  is controlled by the constraints that are passed in which should be an
  instance of the GOGraph::Crawl::Accept class. This constraint information
  is also why I can't use the transitive closure information which is
  precalculated and denormalised in the GO database, as this is calculated
  using any semantic link type.

  GO contains "orphan terms", that is terms which have no "is-a" parent,
  but which are only "part-of" something. In general this is fine, but
  if you are ignoring "part-of" links, based on the Accept class, this
  leaves GO terms which are not traceable to the root node, which is
  undesirable, at least when this class is being used for calculating
  information content measures. So in this case the class synthesizes
  a link to the root node.

  One of the problems with not using the transitive closure information
  in GO database, is that this class is potentially very slow. Its certainly
  a lot slower than using graph iterators direct from the GO AppHandle.
  The solution to this is to cache all of the results in memory, and
  returns these instead. This means that this class can use quite a lot
  of memory. Moreover changing the Accept criteria after instantiation
  is bad idea, and will not work properly.

  One of the other problems with this class is that it doesn't synthesize
  all of the links should do. There is a rule for GO which says that if
  A is a part of B, and C is a B, then A is also part of C (or something
  like that), which should be accounted for when calculating information
  content, but which isn't.

=head1 AUTHOR

  Phillip Lord (p.lord@russet.org.uk)

=head1 COPYRIGHT

  This code is copyright by the University of Manchester (2002)

=head1 LICENSE

  This module is free software; you can redistribute it or
  modify it under the same terms as Perl itself.

  If you use this software for published research, please
  check my website (http://www.russet.org.uk) for recent
  citation.

=cut

use strict;
use Exporter;
use vars qw(@ISA);



sub new{
  my $proto = shift;
  my $class = ref( $proto ) || $proto;
  my $accept = shift;
  my $root_acc = shift;

  my $self =
    {
     database=>0,
     memory=>0,
     path_cache=>{},
     accept=>$accept,
     root_acc=>$root_acc
    };
  bless $self, $class;
  return $self;
}

sub get_accept{
  my $self = shift;
  return $self->{accept};
}

sub get_root_acc{
  my $self = shift;
  return $self->{root_acc} || "root";
}

=head2 crawl_to_root

  This function crawls upwards from any term, via only "isa" links,
  picking up all the terms along the way, and returning them.

  One of the difficulties with the Gene Ontology is the presence
  of "orphan" elements, that is concepts which have no parents (but
  are part-of something else). In this case we work around this problem
  by synthesising fake likes up to the root node.

  Returns - Hash Ref of GO::Model::Term
  Args - Term

=cut
sub crawl_to_root{
  my $self = shift;
  my $term = shift;

  my $apph = $term->apph;

  ## this is the path caching.  In a sense this path cache is in the
  ## wrong place. It should be inside the recursive call really. The
  ## problem is that the call passes the $terms hash. This means that
  ## the recursive call doesn't have at any point the actual hash from
  ## that point upward to cache. I could have the recursive call
  ## return a hash with this, but then I'd have to add one hash to
  ## another repeatedly, and I don't know how to do this
  ## efficiently. It would probably be worth doing even inefficiently,
  ## as the DAG is highly connected, and calculating the transitive
  ## closure is going to be slow if I repeatedly graph over the same
  ## stuff.

  ## how to merge two hashes...
  ## %bighash = ( %hash1, %hash2 );
  my $path_cache = $self->{path_cache};
  my $path_identifier = $term->acc;

  my $cached_path = $path_cache->{$path_identifier};
  if( defined $cached_path ){
    return $cached_path;
  }

  ## now fetch and cache the root term
  unless( defined $self->{root_term} ){
    if( !defined( $self->{root_acc} ) ){
      $self->{root_term} = $apph->get_root_term;
    }
    else{
      $self->{root_term} = $apph->get_term({acc=>$self->{root_acc}});
    }
  }

  my $root = $self->{root_term};

  #if( $self->{database} % 50 == 0 ){
  #  print "Crawl to root cache " . $self->{database} . "\n";
  #}
  my $terms = {};

  $self->_crawl_to_root0
    ( $term, $terms, $root, $apph );

  ## Stuff on root node in case it ain't there.
  $terms->{$root->acc} = $root;

  ## cache the result
  $path_cache->{$path_identifier} = $terms;
  return $terms;
}


sub _crawl_to_root0{
  my $self = shift;
  ## term to crawl from.
  my $term = shift;
  ## terms to add to.
  my $terms = shift;
  my $root_term = shift;
  my $apph = shift;

  #print "\tCrawling to root with " . $self->{accept}->name .
  #  " and " . $term->acc . "\n";

  ## add the term to the termlist. Use the accn number as a hash
  ## because this is unique to each term.
  #print "term is " + $term->acc . "\n";
  $terms->{ $term->acc } = $term;

  ## stop conditions
  ## Have we got to the root.
  if( $term->acc eq $root_term->acc ){
    return $terms;
  }

  ## If not then graph up towards the top where ever possible.
  foreach my $t (@{ $term->get_parent_terms }){
    my $relation = $apph->get_relationships( {child=>$term, parent=>$t} );
    if( $self->{accept}->accept( $relation->[ 0 ] ) ){
      $self->_crawl_to_root0( $t, $terms, $root_term, $apph );
    }
  }

  return $terms;
}


1;
