#!/usr/bin/perl -w

package GOGraph::GeneProduct::SemanticSimilarity::CombinedSimilarity;

=head1 NAME

  GOGraph::GeneProduct::SemanticSimilarity::CombinedSimilarity

=head1 SYNOPSIS

  This class is abstract, and is used only via its subclasses.

  my $combined_similarity
  = GOGraph::GeneProduct::SemanticSimilarity::PairwiseAverageSimilarity
  ->new( GOGraph::Similarity::Factory::resnick_aspect_function_good() );

  my $similarity = $combined_similarity_pairwise->semantic_similarity
  ( $go_terms, $go_terms );


=head1 DESCRIPTION

  This is an abstract base class for combining the similarity of sets
  of GO terms, when at least one of the sets has more than one term.

  Many GeneProducts have more than a single GO annotation. Depending
  on how the similarity is being calculated, often more than than one
  Term will need to be considered when calculating the similarity
  between these GeneProducts.

  There are several different ways in which we can combine
  similarities, and this base class encapsulates these methods.

=head2 new

  Args - an instance of GOGraph::Similarity::Similarity;
  Returns - a new instance of this class

=head2 semantic_similarity

  Args - two arrayrefs of GO::Model::Term objects
  Returns - the similarity between these terms

=head1 AUTHOR

  Phillip Lord (p.lord@russet.org.uk)

=head1 COPYRIGHT

  This code is copyright by the University of Manchester (2002)

=head1 LICENSE

  This module is free software; you can redistribute it or
  modify it under the same terms as Perl itself.

  If you use this software for published research, please
  check my website (http://www.russet.org.uk) for recent
  citation.

=cut

use strict;
use Carp;



sub new{
  my $proto = shift;
  my $class = ref( $proto ) || $proto;

  my $semantic_similarity = shift;

  my $self={};
  bless $self, $class;

  $self->{similarity} = $semantic_similarity;
  $self->_initialize;
  return $self;
}

sub _initialize{}

sub semantic_similarity{
  croak "The semantic similarity method should be overridden by subclasses";
}

1;

