#!/usr/bin/perl -w

package GOGraph::Similarity::InformationContent::Base::CalculateSimilarity;

=head1 NAME

  GOGraph::Similarity::InformationContent::Base::CalculateSimilarity;

=head1 SYNOPSIS

  Just run the script.

=head1 DESCRIPTION

  This script precalculates the information content values for each of
  the GO terms, which is then used by the various information content
  based measures. Currently this class loads all of the data into
  memory, and then dumps everything into the database. Originally I
  tried using the database to store the information, but this was very
  slow.

  To run this script you must have created the database previously,
  with the CreateDB.pl script. This script takes care of clearing the
  database first. The database name, and other information, is read
  from a Config file, see Config::App::AppProperties.

  The information content is calculated by adding up the number of
  occurences of a term, or any of its children. Within GO, there are a
  number of different ways in which you can do this, which are
  encompassed in the "constraints", which are defined by the various
  "Accept" classes, here and in the CrawlToRoot package. Currently
  this class calculates all of the different constraint sets present
  in the GOGraph::Similarity::Factory module. If you don't need all of
  these, you can speed things up by removing some of them. See the in
  code documentation for detail.

=head1 AUTHOR

  Phillip Lord (p.lord@russet.org.uk)

=head1 COPYRIGHT

  This code is copyright by the University of Manchester (2002)

=head1 LICENSE

  This module is free software; you can redistribute it or
  modify it under the same terms as Perl itself.

  If you use this software for published research, please
  check my website (http://www.russet.org.uk) for recent
  citation.

=cut

use strict;
use DBI;


use GOGraph::GOAppHandleFactory;
use GOGraph::Similarity::Factory;
use GOGraph::GeneProduct::Collections::AllUniprotCollection;



my $clear2=<<SQL;
delete from similarity
SQL

my $clear1=<<SQL;
delete from constraint_type
SQL


## Get all the connections most of the connections.
my $go_handle = GOGraph::GOAppHandleFactory::go_apph();

## These are the constraint sets which we are going to use to
## calculate the information content. Calculating all of them takes a
## little bit longer, but makes sure they are all there if I want
## them.
my @constraints = @{GOGraph::Similarity::Factory->constraints_array_reference()};

# @constraints = (
#                 GOGraph::Similarity::Factory->constraint_all_relation_function_terms(),
#                 GOGraph::Similarity::Factory->constraint_all_relation_component_terms(),
#                GOGraph::Similarity::Factory->constraint_all_relation_process_terms() );


my $insert_occur=<<SQL;
insert into similarity values
  ( ?, ?, ?, ? )
SQL

## get the sim connection, create the database. I'm doing all this in
## a routine, so I can renew the connection periodically, in the hope
## that it will clear a memory leak that I am having.
my $sim_connection =
  GOGraph::Similarity::InformationContent::Base::DatabaseFactory::database_handle();
$sim_connection->do( $clear1 );
$sim_connection->do( $clear2 );



### Here we want to translate constraints into type ids
my $_insert_constraint_type=<<SQL;
insert into constraint_type values( ?, ? )
SQL


for(my $i = 0;
    $i < scalar( @constraints );
    $i++ ){
  my $constraint = $constraints[ $i ];
  my $sth = $sim_connection->prepare( $_insert_constraint_type );
  $sth->execute( $i, $constraint->_get_description_from_constraints() );
}


## and create the statements.
my $insert_sth;
_sim_statement_prepare();

#use  GOGraph::GeneProduct::Collections::TestCollection;
#my $swissprot_proteins = GOGraph::GeneProduct::Collections::TestCollection
#    ->new->collection;
my $swissprot_proteins = GOGraph::GeneProduct::Collections::AllUniprotCollection
  ->new->collection;
#use GOGraph::GeneProduct::Collections::AllSGDCollection;
#my $swissprot_proteins = GOGraph::GeneProduct::Collections::AllSGDCollection
#  ->new->collection;

my $size = @$swissprot_proteins;
print "Size of Swissprot is " . $size . "\n";
my $number = 0;

foreach my $swissprot_id ( @$swissprot_proteins ){

  print "On protein " . $number++ . " $swissprot_id\n";
#  if( $number > 300 ){
#    last;
#  }

  ## Sometimes this line crashes with "Odd number of elements in hash assignment".
  ## Actually its a warning. Removing -w from the front stops it from complaining.
  my $terms_from_protein = $go_handle->get_terms({'products'=>$swissprot_id});

  for( my $i = 0;
       $i < scalar( @constraints );
       $i++ ){
    my $constraint = $constraints[ $i ];
    foreach my $term ( @$terms_from_protein ){
      #print "Working on term " . $term->id() . " with constraint " . $constraint->_get_description_from_constraints() . "\n";
      if( $constraint->get_accept_term->accept( $term ) ){
      #  print "for constraint " . $constraint->_get_description_from_constraints() . " \n";
        update_all_terms
          (
           $constraint->get_crawler->crawl_to_root( $term ),
           $i
          );
      }
      else{
       # print "no terms for " . $constraint->_get_description_from_constraints() . "\n";
      }
    }
  }
}

sync_to_database();

$insert_sth->finish;


$sim_connection->disconnect() || die "$!";

exit( 0 );


sub _sim_statement_prepare{
  $insert_sth = $sim_connection->prepare( $insert_occur );
}

sub update_all_terms{
  my $terms = shift;
  my $type_id = shift;

  foreach my $term( values( %$terms ) ){
    ##print "\t\tUpdating term " . $term->acc . "\n";
    update_term( $term, $type_id );
  }
}

my $similarity_id_value = 0;
my $update_count;

## This is a two dimensional hash. The first dimension is for the type
## id's, and the second contains all the terms, with their number of
## occurences.
my $occurence_hash_of_hash = {};

sub update_term{
  my $term = shift;
  my $acc = $term->acc;
  my $type_id = shift;

  my $occurence_hash = $occurence_hash_of_hash->{$type_id};

  if( !defined( $occurence_hash ) ){
    $occurence_hash = {};
    $occurence_hash_of_hash->{$type_id} = $occurence_hash;
  }
  $occurence_hash->{ $acc }++;
}


sub sync_to_database{
  foreach my $type_id (keys %$occurence_hash_of_hash){
    my $occurence_hash = $occurence_hash_of_hash->{$type_id};
    foreach my $acc (keys %$occurence_hash){
      print "GO term " . $acc . " occurred " . $occurence_hash->{$acc} .
        " times with type id of " . $type_id . "\n";
      $insert_sth->execute
        ( $similarity_id_value++, $acc, $type_id, $occurence_hash->{$acc} );
    }
  }
}
