# !/usr/bin/perl -w

package GOGraph::Similarity::InformationContent::Base::InformationContentSimilarity;

=head1 NAME

  GOGraph::Similarity::InformationContent::Base::InformationContentSimilarity;

=head1 SYNOPSIS

  my $similarity = GOGraph::Similarity::Factory::resnick_sim_all();
  my $sim = $similarity->similarity_between_terms( $term_one, $term_two );

=head1 DESCRIPTION

  An abstract class to calculate the information content based
  similarity betweeen two GO terms. It calculates the probability of
  the minimum subsumer, which is used by all information content based
  similarity measures, and then leaves the rest to the subclasses.

  Generally speaking subclasses of this class will be created through
  the Factory class, as shown above. If you want to create Similarity
  objects, then examples can be found in this class.

=head1 AUTHOR

  Phillip Lord (p.lord@russet.org.uk)

=head1 COPYRIGHT

  This code is copyright by the University of Manchester (2002)

=head1 LICENSE

  This module is free software; you can redistribute it or
  modify it under the same terms as Perl itself.

  If you use this software for published research, please
  check my website (http://www.russet.org.uk) for recent
  citation.

=cut

use strict;
use vars qw{@ISA};
use Carp;

sub new{
  my $proto = shift;
  my $class = ref($proto) || $proto;

  my $self = {};
  bless $self, $class;

  $self->_initialize(@_);
  return $self;
}


sub _initialize{
  my $self = shift;
  my $constraints = shift;

  $self->{constraints} = $constraints;
  $self->{crawler} = $constraints->get_crawler;
  $self->{accept} = $constraints->get_accept_term;

  my $occurrence =
    GOGraph::Similarity::InformationContent::Base::OccurrenceFactory
        ->create_instance();
  $self->{occurrence} = $occurrence;
  $self->{constraint_name} = $self->{constraints}
    ->_get_description_from_constraints();
  $self->{similarity_cache} ={};
}

sub constraint_name{
  my $self = shift;
  return $self->{constraint_name};
}

sub similarity_between_terms{
  my $self = shift;
  my $first_term = shift;
  my $second_term = shift;

  if( !($self->{accept}->accept( $first_term )
        && $self->{accept}->accept( $second_term )) ) {
    return -1;
  }

  #my $cache = $self->{similarity_cache};
  #my $ident = $first_term->acc . $second_term->acc;
  #my $sim = $cache->{$ident};
  #if( defined $sim ){
  #  return $sim;
  #}

  ## crawl to the root from each term
  my $first_subsumptions = $self->{crawler}->crawl_to_root( $first_term );
  my $second_subsumptions = $self->{crawler}->crawl_to_root( $second_term );

  ## fetch any common subsumptions for between these two sets of terms
  my $common_subsumptions = $self->_common_subsumptions( $first_subsumptions, $second_subsumptions );
  ## which of these has the highest information content?
  my $rarest_occurrence = $self->_rarest_occurrence( $common_subsumptions );
  ## calculate the similarity.
  my $sim = $self->_similarity( $rarest_occurrence,
                             $first_term, $second_term );
  #$cache->{$ident} = $sim;
  return $sim;
}





## this method finds all common subsumptions between the two groups
## of terms.
sub _common_subsumptions{
  my $self = shift;
  my $first_hash = shift;
  my $second_hash = shift;

  my @common;
  foreach my $accn ( keys( %$first_hash ) ){
    if( defined( $second_hash->{$accn} ) ){
      push( @common, $second_hash->{$accn} );
    }
  }
  return \@common;
}


sub _rarest_occurrence{
  my $self = shift;
  my $terms = shift;

  ## Start off with a high value. Add the 1 in case the top GO node is
  ## rarest, $rarest_term will not get set.
  my $rarest_occurrence = $self->{occurrence}->fetch_max_occurrences
    ( $self->{constraint_name} ) + 1;
  my $rarest_term;

  ## iterate through each term, to see if it occurs less frequently
  ## than the rarest so far.
  foreach my $term ( @$terms ){
    my $go_accn = $term->acc;
    my $occurrences =
      $self->{occurrence}->fetch_occurrences
        ( $self->{constraint_name}, $go_accn );

    if( $rarest_occurrence > $occurrences ){
      $rarest_occurrence = $occurrences;
      $rarest_term = $term;
    }
  }
  return $rarest_occurrence;
}


1;
