#!/usr/bin/perl -w

package GOGraph::Similarity::InformationContent::Base::SqlOccurrence;

=head1 NAME

  GOGraph::Similarity::InformationContent::Base::SqlOccurrence

=head1 DESCRIPTION

  This class implements the Occurrence interface, and operates as a
  thin wrapper around the information content database, which is
  filled using the CalculateSimilarity script.

=head1 AUTHOR

  Phillip Lord (p.lord@russet.org.uk)

=head1 COPYRIGHT

  This code is copyright by the University of Manchester (2002)

=head1 LICENSE

  This module is free software; you can redistribute it or
  modify it under the same terms as Perl itself.

  If you use this software for published research, please
  check my website (http://www.russet.org.uk) for recent
  citation.

=cut

use strict;
use Exporter;
use vars qw{@ISA};
use Carp;

use DBI;

use GOGraph::Similarity::InformationContent::Base::DatabaseFactory;
use base qw{GOGraph::Similarity::InformationContent::Base::Occurrence};





my $_fetch_constraint_type=<<SQL;
select id from constraint_type where short_name = ?
SQL

my $_fetch_max=<<SQL;
select max( occurences ) from similarity where type_id = ?
SQL

my $_fetch_occurence=<<SQL;
select occurences from similarity where go_accn = ? and type_id = ?
SQL


sub _initialize{
  my $self = shift;

  $self->{connection} = GOGraph::Similarity::InformationContent::Base::DatabaseFactory::database_handle();

  ## This is where we store the data about the individual constraint type data
  ## that we want to cache. In this case this is the type_id, and the max
  ## number of occurences for that constraint_type.

  ## The data structure is a hash of hashs. The first dimension has key
  ## of the constraint name, and value of the hash. This hash has key of
  ## "max" and "id" with values of, well, the max and the id.
  $self->{constraint_data} = {};

  ## prepare statements.
  $self->{type_id_sth} = $self->{connection}->prepare( $_fetch_constraint_type );
  $self->{fetch_max_sth} = $self->{connection}->prepare( $_fetch_max );
  $self->{fetch_occurrence_sth} = $self->{connection}->prepare( $_fetch_occurence );
}


## returns a hash with the constraint data in it....
sub _fetch_constraint_data{
  my $self = shift;
  my $constraint_name = shift;

  ## have we got this already
  my $constraint_hash = $self->{constraint_data}->{$constraint_name};
  if( defined( $constraint_hash ) ){
    return $self->{constraint_data}->{$constraint_name};
  }

  ## no? then gather the data.
  ## probably there should be some failure conditions here.
  $constraint_hash = {};
  my $type_sth = $self->{type_id_sth};

  $type_sth->execute
    ( $constraint_name ) || croak "$!";

  $constraint_hash->{id} = $type_sth->fetchrow_arrayref->[ 0 ];

  my $fetch_max_sth = $self->{fetch_max_sth};
  $fetch_max_sth->execute( $constraint_hash->{id} ) || croak "$!";
  $constraint_hash->{max} = $fetch_max_sth->fetchrow_arrayref->[ 0 ];

  return $constraint_hash;
}

## These methods are the interface specification.
sub fetch_max_occurrences{
  my $self = shift;
  my $constraint_name = shift;

  my $constraint_data = $self->_fetch_constraint_data( $constraint_name );
  return $constraint_data->{max};
}

sub fetch_occurrences{
  my $self = shift;
  my $constraint_name = shift;
  my $goaccn = shift;

  my $constraint_data = $self->_fetch_constraint_data( $constraint_name );

  $self->{fetch_occurrence_sth}->execute
    ( $goaccn, $constraint_data->{id} ) || croak "$!";
  my $results = $self->{fetch_occurrence_sth}->fetchrow_arrayref;
  unless( defined $results->[ 0 ] ){
    return 0;
  }
  return $results->[0];
}

sub DESTROY{
  my $self = shift;

  eval{$self->{type_id_sth}->finish};
  eval{$self->{fetch_max_sth}->finish};
  eval{$self->{fetch_occurrence_sth}->finish};
}


1;

