#!/usr/bin/perl -w

package GOGraph::Statistics::GOCompare;

=head1 NAME

  GOGraph::Statistics::GOCompare

=head1 SYNOPSIS

  my $printout = GOGraph::Statistics::PrintSortedResults->new();

  my $goaspect = GOGraph::Statistics::GOCompare->new
    ( $comparitors, $printout );

  $goaspect->compare( $query_protein, $search_protein );

=head1 DESCRIPTION

  This class provides a wrapper for doing semantic comparions between proteins.
  You can pass in any number of comparitors, and a prototype output handler.
  The compare method will then compare the two proteins with the various
  comparitors, sending the result to the output handler.

=cut

use GOGraph::GeneProduct::SemanticSimilarity::PairwiseAverageSimilarity;
use GOGraph::GeneProduct::SemanticSimilarity::GeneProductSimilarity;
use GOGraph::Statistics::PrintOutResults;

use strict;
use Carp;



sub new{
  my $proto = shift;
  my $class = ref( $proto ) || $proto;

  my $sem_comparisons = shift;
  if( !defined $sem_comparisons ){
    croak "Must pass in comparitors to GOCompare";
  }

  my $output_proto = shift || GOGraph::Statistics::PrintOutResults->new();
  my $outputs = {};
  my $swissprots = {};
  foreach my $comparitor( @$sem_comparisons ){
    my $comparitor_name = $comparitor->constraint_name();
    my $output = $output_proto->new( $comparitor_name );
    $outputs->{$comparitor_name} = $output;
    my $combined_sim = GOGraph::GeneProduct::SemanticSimilarity::PairwiseAverageSimilarity->new
      ( $comparitor );
    my $swissprot =
      GOGraph::GeneProduct::SemanticSimilarity::GeneProductSimilarity->new( $combined_sim );
    $swissprots->{$comparitor_name} = $swissprot;
  }
  my $self =
    {
     comparitors=>$sem_comparisons,
     swissprot=>$swissprots,
     output=>$outputs
    };
  bless $self, $class;
  return $self;
}


sub compare{
  my $self = shift;
  my $acc = shift;
  my $match_acc = shift;
  my $subject = shift;

  foreach my $comparitor( @{$self->{comparitors}} ){
    my $comparitor_name = $comparitor->constraint_name();

    my $swissprot = $self->{swissprot}->{$comparitor_name};
    my $output = $self->{output}->{$comparitor_name};

    my $sem_sim = $swissprot->semantic_similarity( $acc, $match_acc );
    $output->process_result( $sem_sim, $acc, $match_acc, $subject );
  }
}


1;
